import 'package:flutter/widgets.dart' as f;
import 'package:particle_ui/particle.dart';

enum TextType {
  headline1, headline2, headline3, headline4, headline5, headline6, subtitle1, subtitle2, body1, body2, button, footnote, caption1, caption2
}

class Text extends StatelessWidget {
  final String data;

  final TextType type;
  final Font font;
  final TextAlign textAlign;
  final Fill fill;
  final EdgeInsets padding;
  final Alignment alignment;
  final int maxLines;
  final TextOverflow overflow;

  const Text(this.data, {Key key, this.type = TextType.body1, this.font, this.textAlign, this.fill, this.padding, this.alignment, this.maxLines, this.overflow}) : assert(type != null), super(key: key);

  const Text.headline1(this.data, {Key key, this.textAlign, this.fill, this.padding, this.alignment, this.maxLines, this.overflow}) : type = TextType.headline1, font = null, super(key: key);
  const Text.headline2(this.data, {Key key, this.textAlign, this.fill, this.padding, this.alignment, this.maxLines, this.overflow}) : type = TextType.headline2, font = null, super(key: key);
  const Text.headline3(this.data, {Key key, this.textAlign, this.fill, this.padding, this.alignment, this.maxLines, this.overflow}) : type = TextType.headline3, font = null, super(key: key);
  const Text.headline4(this.data, {Key key, this.textAlign, this.fill, this.padding, this.alignment, this.maxLines, this.overflow}) : type = TextType.headline4, font = null, super(key: key);
  const Text.headline5(this.data, {Key key, this.textAlign, this.fill, this.padding, this.alignment, this.maxLines, this.overflow}) : type = TextType.headline5, font = null, super(key: key);
  const Text.headline6(this.data, {Key key, this.textAlign, this.fill, this.padding, this.alignment, this.maxLines, this.overflow}) : type = TextType.headline6, font = null, super(key: key);
  const Text.subtitle1(this.data, {Key key, this.textAlign, this.fill, this.padding, this.alignment, this.maxLines, this.overflow}) : type = TextType.subtitle1, font = null, super(key: key);
  const Text.subtitle2(this.data, {Key key, this.textAlign, this.fill, this.padding, this.alignment, this.maxLines, this.overflow}) : type = TextType.subtitle2, font = null, super(key: key);
  const Text.body1(this.data, {Key key, this.textAlign, this.fill, this.padding, this.alignment, this.maxLines, this.overflow}) : type = TextType.body1, font = null, super(key: key);
  const Text.body2(this.data, {Key key, this.textAlign, this.fill, this.padding, this.alignment, this.maxLines, this.overflow}) : type = TextType.body2, font = null, super(key: key);
  const Text.button(this.data, {Key key, this.textAlign, this.fill, this.padding, this.alignment, this.maxLines, this.overflow}) : type = TextType.button, font = null, super(key: key);
  const Text.footnote(this.data, {Key key, this.textAlign, this.fill, this.padding, this.alignment, this.maxLines, this.overflow}) : type = TextType.footnote, font = null, super(key: key);
  const Text.caption1(this.data, {Key key, this.textAlign, this.fill, this.padding, this.alignment, this.maxLines, this.overflow}) : type = TextType.caption1, font = null, super(key: key);
  const Text.caption2(this.data, {Key key, this.textAlign, this.fill, this.padding, this.alignment, this.maxLines, this.overflow}) : type = TextType.caption2, font = null, super(key: key);

  @override
  Widget build(BuildContext context) {
    Text _theme = Theme.of(context).texts[type];
    assert(_theme != null);

    Text _property = mergeWith(_theme);

    Widget _widget;

    Color _color = _property.fill;
    f.TextStyle _textStyle;
    if (_property.font != null && _property.fill != null) {
      _textStyle = f.TextStyle(
        fontFamily: _property.font.fontFamily,
        fontSize: _property.font.fontSize,
        fontWeight: _property.font.fontWeight,
        fontStyle: _property.font.fontStyle,
        letterSpacing: _property.font.letterSpacing,
        wordSpacing: _property.font.wordSpacing,
        height: _property.font.height,
        color: _color.to32BitARGBColor()
      );
      _widget = f.Text(data, style: _textStyle, textAlign: _property.textAlign, overflow: _property.overflow, maxLines: _property.maxLines);
    } else {
      _widget = f.Text(data, style: f.TextStyle(color: _color.to32BitARGBColor()), textAlign: _property.textAlign, overflow: _property.overflow, maxLines: _property.maxLines);
    }

    if (_property.padding != null) {
      _widget = f.Padding(padding: _property.padding, child: _widget);
    }

    if (_property.alignment != null) {
      _widget = f.Align(alignment: _property.alignment, child: _widget);
    }

    return _widget;
  }

  Text mergeWith(Text text) {
    if (text != null) {
      return Text(
        data,
        font: font ?? text.font,
        textAlign: textAlign ?? text.textAlign,
        fill: fill ?? text.fill,
        padding: padding ?? text.padding,
        alignment: alignment ?? text.alignment,
        maxLines: maxLines ?? text.maxLines,
        overflow: overflow ?? text.overflow
      );
    } else {
      return this;
    }
  }
}